#include <stdio.h>
#include <string.h>
#include "filmbuf.h"

static Buffer *filmbuf=NULL;

char *Filmbuf(void) {
	return filmbuf->buf;
	}

void ClearFilmbuf(void) {
	if (filmbuf!=NULL)
		ClearBuffer(filmbuf);
	}

void AddToFilmbuf(char *str) {
	if (filmbuf==NULL)
		filmbuf=CreateBuffer("film", 8192, 4096, 2000000);

	AddToBuffer(filmbuf, str);
	}

void AddToFilmbuf2(char *str1, char *str2) {
	AddToFilmbuf(str1);
	AddToFilmbuf(str2);
	}

void AddToFilmbuf3(char *str1, char *str2, char *str3) {
	AddToFilmbuf(str1);
	AddToFilmbuf(str2);
	AddToFilmbuf(str3);
	}


